----------------------------------------------------------------------------------
-- Company: 		 Hobby
-- Engineer: 		 Juha Linna
-- 
-- Create Date:    19:53:37 02/11/2011 
-- Design Name: 
-- Module Name:    ATM_top - Behavioral 
-- Project Name: 	 ATM_MuxAndReg
-- Target Devices: X2C128 100pin
-- Tool versions:  ISE 10.1
-- Description: 	 Atmel AVR ATMega128 AD muxing and Led register
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity ATM_top is
	Port (
		Rst   : in STD_LOGIC;
		AD0_7 : inout  STD_LOGIC_VECTOR (7 downto 0);
		A8_15 : in  STD_LOGIC_VECTOR (7 downto 0);
		A0_7  : out  STD_LOGIC_VECTOR (7 downto 0);
		Leds1_0_7 : out  STD_LOGIC_VECTOR (7 downto 0);
		Leds2_0_7 : out  STD_LOGIC_VECTOR (7 downto 0);
		Dips1_0_7 : in  STD_LOGIC_VECTOR (7 downto 0);
		Dips2_0_7 : in  STD_LOGIC_VECTOR (7 downto 0);
		CE0 : out STD_LOGIC;
		CE1 : out STD_LOGIC;
		CE2 : out STD_LOGIC;
		CE3 : out STD_LOGIC;			  
		ALE : in  STD_LOGIC;
		WE  : in  STD_LOGIC;
		RD  : in  STD_LOGIC);
end ATM_top;

architecture Behavioral of ATM_top is
	signal A0_7_i:   STD_LOGIC_VECTOR (7 downto 0);
	signal AD0_7_o:  STD_LOGIC_VECTOR (7 downto 0);
	signal CE_Leds: STD_LOGIC;
	signal CE_Dips: STD_LOGIC;
	signal CE1_i: STD_LOGIC;
	signal CE2_i: STD_LOGIC;
	signal CE3_i: STD_LOGIC;
begin


-- AD Mux register
  process(ALE, Rst, AD0_7) 
  begin
		if Rst = '0' then
			A0_7_i <= (others => '0');
		else
			if rising_edge(ALE) then      
				A0_7_i <=  AD0_7;
			end if;
		end if;
  end process;

-- Led buffers
  process(WE, RD, Rst, CE_Leds, A0_7_i, AD0_7)
  begin			
		if Rst = '0' then
			Leds1_0_7 <= (others => '0');
			Leds2_0_7 <= (others => '0');
		elsif rising_edge(WE) then
			if (CE_Leds = '1') and RD = '0' then
				if A0_7_i(0) = '0' then -- Addr 0xFFFE
					Leds1_0_7 <= AD0_7;
				elsif A0_7_i(0) = '1' then -- Addr 0xFFFF
					Leds2_0_7 <= AD0_7;
				end if;
			end if;
		end if;
  end process;
	
-- Dip switch inputs
	process(WE, RD, Rst, Dips1_0_7)
	begin		 
		if Rst = '0' then
			AD0_7_o <= (others => '0');
		elsif rising_edge(RD) then
			if CE_Dips = '1' and WE = '0' then
				if A0_7_i(0) = '0' then -- Addr 0xFFFE
					AD0_7_o <= Dips1_0_7;
				elsif A0_7_i(0) = '1' then -- Addr 0xFFFF
					AD0_7_o <= Dips2_0_7;
				end if;
			end if;
		end if;
	end process;


	-- Asyncronous signals !!!
	-- CE for RAM
	CE0 <= not ( CE1_i or (CE2_i or (CE3_i or (CE_Leds or CE_Dips))));

	-- Device Bloxs 256B
	CE1_i <= '1' when A8_15 = "10111111" else '0'; -- 0xFD00 to 0xFDFF
	CE2_i <= '1' when A8_15 = "01111111" else '0'; -- 0xFE00 to 0xFEFF
	CE3_i <= '1' when A8_15 = "11111111" and CE_Leds = '0' and CE_Dips = '0' else '0'; -- 0xFF00 to 0xFFFE

	-- Leds Chip enable 0xFFFC to 0xFFFF
	CE_Leds <= '1' when A8_15 = "11111111" and A0_7_i(7 downto 1) = "1111111"  else '0'; -- 0xFFFF - 0xFFFE
	CE_Dips <= '1' when A8_15 = "11111111" and A0_7_i(7 downto 1) = "1111110"  else '0'; -- 0xFFFD - 0xFFFC
	
	-- Map to output pins
	CE1 <= CE1_i;
	CE2 <= CE2_i;
	CE3 <= CE3_i;
	A0_7 <= A0_7_i;

	-- Map inout pin
	AD0_7 <= AD0_7_o when RD = '1' and Rst = '1' and CE_Dips = '1' else (others => 'Z');
  
end Behavioral;

